/*
 * file:	terraced_11.pov
 * desc:	A "terraced world" based on a macro who performs slices of an object using prisms.
 * auth:	Paolo Gibellini
 * date:	2006-05-25
 * todo:	Improve or change the macro (problems with some objects, works only on a external envelope).
 *			Actually is a very simple algorithm, could be performed by using a circular generation
 *			of the normal (eventually rotating the object).
 *			Instead of using a prism could be used a repeated intersection of the object with a thin box
 */

#include "textures.inc" 
#include "functions.inc"
#include "transforms.inc"

// Macro ---------------------------------------------------------------------
/*
 * macro:		slices(OBJ_IN,n_slices,iimax,dist)
 * descr:		Parse an object and create slices with external profile of the object
 * param:		OBJ_IN			Object to be sliced
 * param:		n_slices		Number of slices
 * param:		iimax			Half of the number of the segments of the prism
 * param:		dist 			Distance between slices
 */
#macro slices(OBJ_IN,n_slices,iimax,dist)
	#local xi=(max_extent(OBJ_IN).x+min_extent(OBJ_IN).x)/2; 
	#local zi=(max_extent(OBJ_IN).z+min_extent(OBJ_IN).z)/2; 
	#local xd=(max_extent(OBJ_IN).x-min_extent(OBJ_IN).x)/2;  
	#local zd=(max_extent(OBJ_IN).z-min_extent(OBJ_IN).z)/2;
	#local r_max=max(xd*2,zd*2); // Da cui sparare le normali
	 
	#local h_max=max_extent(OBJ_IN).y-min_extent(OBJ_IN).y; // Scansione verticale
	#local i=h_max/n_slices;
	#local yi=min_extent(OBJ_IN).y;
	#local N = <0,0,0>;
	#while(i<h_max)
		#local ii=0;
		#local old_x=min_extent(OBJ_IN).x;
		#local old_z=min_extent(OBJ_IN).z;
		#local Pigm=pigment {rgb 1};
		prism{
			linear_spline
			yi,
			yi+(h_max/n_slices-dist),
			iimax*2+2,
			
			// First point, x direction
			#local S = trace(OBJ_IN, <xi-2*xd,yi,zi>, x, N);
			#if (vlength(N)>0) //hit
				//#local Pigm=eval_pigment(MyPigment, S);
				#local old_x=S.x;
				#local old_z=S.z;
				<S.x,S.z>
			#else // default
				<old_x,old_z>
				//<0,0>
			#end
			// iimax points, z direction
			#local xc=min_extent(OBJ_IN).x;
			#while (ii<iimax)
				#local S = trace(OBJ_IN, <xc,yi,zi-2*zd>, z, N);
				#local xc=xc+2*xd/iimax;
				#if (vlength(N)>0) //hit
					//#local Pigm=eval_pigment(MyPigment, S);
					#local old_x=S.x;
					#local old_z=S.z;
					<S.x,S.z>
				#else // default
					<old_x,old_z>
					//<0,0>
				#end
				#local ii=ii+1;
				#if(ii<iimax) // Comma in order to separe values of the spline
					,
				#end
			#end
			// Middle point, -x direction
			#local S = trace(OBJ_IN, <xi+2*xd,yi,zi>, -x, N);
			#if (vlength(N)>0) //hit
				//#local Pigm=eval_pigment(MyPigment, S);
				#local old_x=S.x;
				#local old_z=S.z;
				<S.x,S.z>
			#else // default
				<old_x,old_z>
				//<0,0>
			#end
			// iimax points, -z direction
			#local xc=max_extent(OBJ_IN).x;
			#while (ii>0)
				#local S = trace(OBJ_IN, <xc,yi,zi+2*zd>, -z, N);
				#local xc=xc-2*xd/iimax;
				#if (vlength(N)>0) //hit
					//#local Pigm=eval_pigment(MyPigment, S);
					#local old_x=S.x;
					#local old_z=S.z;
					<S.x,S.z>
				#else // default
					<old_x,old_z>
					//<0,0>
				#end
				#local ii=ii-1;
				#if(ii>0) // Comma in order to separe values of the spline
					,
				#end
			#end
			
			pigment {Pigm}
		}
		#local i=i+(h_max/n_slices); 
		#local yi=min_extent(OBJ_IN).y+i;
	#end
#end
// ---------------------------------------------------------------------------

// Sample scene --------------------------------------------------------------
global_settings {
  	assumed_gamma 1.0
}

camera{
    location <0, 3, -5>
    look_at <0,1,0>
    angle 80
}

light_source{ <1500,2500,-2500> color rgb <1,1,1> }

sky_sphere {
  pigment {
    gradient y
    color_map {
      [0.0 rgb <0.6,0.7,1.0>]
      [0.7 rgb <0.0,0.1,0.8>]
    }
  }
}

plane{ <0,1,0>, 0 
      texture{ 
		pigment { 
         image_map { 
            function 10,10 { 
               pigment { checker 1,0 scale .5  }
            }
         }
         rotate x*90
      	} 
     } 
}   

// Texture
#declare MyPigment=pigment {rgb 1};

// Object to be parsed
#declare OBJ_IN=union {
	sphere {<0,0,0> 1 texture{pigment{color rgbf <0.8, 0.7, 0.1,0.8> }}}
	cylinder {<0,0,-1.5> <0,0,1.5> 0.5 texture{pigment{color rgbf <0.8, 0.1, 0.1,0.8> }}}
	cylinder {<-1.5,0,0> <1.5,0,0> 0.5 texture{pigment{color rgbf <0.8, 0.1, 0.1,0.8> }}}
	scale 2
	pigment{MyPigment}
}
/*   
#declare OBJ_IN=union {
	torus { 1, .4}
	sphere {<0,0,0> 0.8}
	cylinder {<-1.5,0,0> <1.5,0,0> 0.5}
	scale 2
	pigment{MyPigment}
}     */

slices(OBJ_IN, 20, 30,0.05)
//object {OBJ_IN pigment {rgbf <1,0,0,0.8>}}
// ---------------------------------------------------------------------------
